@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: command.c,v 1.230.2.2 2012/02/25 12:02:06 juhaszp Exp $"); }
+static char *RCSid() { return RCSid("$Id: command.c,v 1.230.2.9 2013/02/11 00:53:14 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - command.c */
@@ -375,11 +375,6 @@ do_line()
 	return (0);
     }
 
-#if 0
-    /* EAM May 2011 */
-    /* Resetting here prevents handling "else" on a separate line */
-    if_condition = TRUE;
-#endif
     if_depth = 0;
     num_tokens = scanner(&gp_input_line, &gp_input_line_len);
 
@@ -460,7 +455,7 @@ do_string_and_free(char *cmdline)
 	extend_input_line();
     strcpy(gp_input_line, cmdline);
     screen_ok = FALSE;
-    do_line();
+    command_exit_status = do_line();
 
     /* We don't know if screen_ok is appropriate so leave it FALSE. */
     lf_pop();
@@ -1150,10 +1145,13 @@ do_command()
     clause_depth++;
     c_token--;	 /* Let the parser see the closing curly brace */
 
-    clause = gp_alloc(do_end - do_start, "clause");
+    clause = gp_alloc(do_end - do_start + 2, "clause");
     memcpy(clause, &gp_input_line[do_start+1], do_end - do_start);
     clause[do_end - do_start - 1] = '\0';
 
+    if (empty_iteration(do_iterator))
+	strcpy(clause, ";");
+
     do {
 	do_string(clause);
     } while (next_iteration(do_iterator));
@@ -1390,7 +1388,7 @@ pause_command()
 	    if (paused_for_mouse && !GraphHasWindow(graphwin)) {
 		if (interactive) { /* cannot wait for Enter in a non-interactive session without the graph window */
 		    if (buf) fprintf(stderr,"%s\n", buf);
-		    fgets(buf, sizeof(buf), stdin); /* graphical window not yet initialized, wait for any key here */
+		    fgets(buf, strlen(buf), stdin); /* graphical window not yet initialized, wait for any key here */
 		}
 	    } else { /* pausing via graphical windows */
 		int tmp = paused_for_mouse;
@@ -1437,7 +1435,7 @@ pause_command()
 	    term->waitforinput();
 	} else
 #endif /* USE_MOUSE */
-	(void) fgets(buf, sizeof(buf), stdin);
+	(void) fgets(buf, strlen(buf), stdin);
 
 #endif /* !(_Windows || OS2 || _Macintosh) */
     }
@@ -2062,6 +2060,7 @@ update_command()
 void
 invalid_command()
 {
+    int save_token = c_token;
 #ifdef OS2
    if (token[c_token].is_token) {
       int rc;
@@ -2073,7 +2072,11 @@ invalid_command()
       }
     }
 #endif
-    int_error(c_token, "invalid command");
+    /* Skip the rest of the command; otherwise we're left pointing to */
+    /* the middle of a command we already know is not valid.          */
+    while (!END_OF_COMMAND)
+	c_token++;
+    int_error(save_token, "invalid command");
 }
 
 
@@ -2163,7 +2166,9 @@ replotrequest()
 	free(replot_args);
     }
     plot_token = 0;		/* whole line to be saved as replot line */
+#ifdef VOLATILE_REFRESH
     refresh_ok = 0;		/* start of replot will destory existing data */
+#endif
 
     screen_ok = FALSE;
     num_tokens = scanner(&gp_input_line, &gp_input_line_len);
@@ -2688,9 +2693,14 @@ rlgets(char *s, size_t n, const char *prompt)
 #  elif defined(HAVE_LIBEDITLINE)
 	    /* deleting history entries does not work, so suppress adjacent 
 	    duplicates only */
-	    while (previous_history());
-	    if (strcmp(current_history()->line, line) != 0)
-		add_history(line);
+      
+	    int found;
+	    using_history();
+
+	    found = history_search(line, -1);
+	    if (found <= 0) {
+               add_history(line);
+            }
 #  else /* builtin readline */
 	    add_history(line);
 #  endif
@@ -2872,6 +2882,16 @@ read_line(const char *prompt, int start)
 
     current_prompt = prompt;	/* HBB NEW 20040727 */
 
+    /* Once we start to read a new line, the tokens pointing into the old */
+    /* line are no longer valid.  We used to _not_ clear things here, but */
+    /* that lead to errors when a mouse-triggered replot request came in  */
+    /* while a new line was being read.   Bug 3602388 Feb 2013.           */
+    /* FIXME: If this causes problems, push it down into fgets_ipc().     */
+    if (start == 0 && !interactive) {
+	c_token = num_tokens = 0;
+	gp_input_line[0] = '\0';
+    }
+
     do {
 	/* grab some input */
 	if (gp_get_string(gp_input_line + start, gp_input_line_len - start,
@@ -3141,7 +3161,7 @@ do_system_func(const char *cmd, char **output)
 
 #else /* VMS || PIPES */
 
-    int_warn(NO_CARET, "system evaluation not supported by %s", OS);
+    int_warn(NO_CARET, "system() requires support for pipes");
     *output = gp_strdup("");
     return 0;
 
