@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: internal.c,v 1.51.2.4 2011/02/17 20:02:45 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: internal.c,v 1.67.2.1 2012/05/06 05:02:58 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - internal.c */
@@ -40,7 +40,7 @@ static char *RCSid() { return RCSid("$Id: internal.c,v 1.51.2.4 2011/02/17 20:02
 #include "stdfn.h"
 #include "alloc.h"
 #include "util.h"		/* for int_error() */
-# include "gp_time.h"           /* for str(p|f)time */
+#include "gp_time.h"           /* for str(p|f)time */
 #include "command.h"            /* for do_system_func */
 #include "variable.h" /* For locale handling */
 
@@ -205,6 +205,54 @@ f_calln(union argument *x)
 }
 
 
+void
+f_sum(union argument *arg)
+{
+    struct value beg, end, varname; /* [<var> = <start>:<end>] */
+    udft_entry *udf;                /* function to evaluate */
+    udvt_entry *udv;                /* iteration variable */
+    struct value ret;               /* result */
+    struct value z;
+    int i;
+
+    (void) pop(&end);
+    (void) pop(&beg);
+    (void) pop(&varname);
+
+    if (beg.type != INTGR || end.type != INTGR)
+        int_error(NO_CARET, "range specifiers of sum must have integer values");
+    if (varname.type != STRING)
+        int_error(NO_CARET, "internal error: f_sum expects argument (varname) of type string.");
+
+    udv = get_udv_by_name(varname.v.string_val);
+    if (!udv)
+        int_error(NO_CARET, "internal error: f_sum could not access iteration variable.");
+    udv->udv_undef = false;
+
+    udf = arg->udf_arg;
+    if (!udf)
+        int_error(NO_CARET, "internal error: f_sum could not access summation coefficient function");
+
+    Gcomplex(&ret, 0, 0);
+    for (i=beg.v.int_val; i<=end.v.int_val; ++i) {
+        double x, y;
+
+        /* calculate f_i = f() with user defined variable i */
+        Ginteger(&udv->udv_value, i);
+        execute_at(udf->at);
+
+        pop(&z);
+        x = real(&ret) + real(&z);
+        y = imag(&ret) + imag(&z);
+        Gcomplex(&ret, x, y);
+    }
+
+    gpfree_string(&varname);
+
+    push(Gcomplex(&z, real(&ret), imag(&ret)));
+}
+
+
 void
 f_lnot(union argument *arg)
 {
@@ -845,6 +893,7 @@ f_power(union argument *arg)
 		(void) Ginteger(&result, b.v.int_val == 0 ? 1 : 0);
 		break;
 	    }
+	    /* EAM Oct 2009 - avoid integer overflow by switching to double */
 	    mag = pow((double)a.v.int_val,(double)b.v.int_val);
 	    if (mag > (double)INT_MAX  ||  b.v.int_val < 0) {
 		(void) Gcomplex(&result, mag, 0.0);
@@ -1046,7 +1095,7 @@ f_strlen(union argument *arg)
     if (a.type != STRING)
 	int_error(NO_CARET, "internal error : strlen of non-STRING argument");
 
-    (void) Ginteger(&result, (int)strlen(a.v.string_val));
+    (void) Ginteger(&result, (int)gp_strlen(a.v.string_val));
     gpfree_string(&a);
     push(&result);
 }
@@ -1071,11 +1120,16 @@ f_strstrt(union argument *arg)
     push(&result);
 }
 
+/*
+ * f_range() handles both explicit calls to substr(string, beg, end)
+ * and the short form string[beg:end].  The calls to gp_strlen() and
+ * gp_strchrn() allow it to handle utf8 strings.
+ */
 void
 f_range(union argument *arg)
 {
     struct value beg, end, full;
-    struct value substr = {0};
+    struct value substr;
 
     (void) arg;			/* avoid -Wunused warning */
     (void) pop(&end);
@@ -1090,15 +1144,19 @@ f_range(union argument *arg)
 
     FPRINTF((stderr,"f_range( \"%s\", %d, %d)\n", full.v.string_val, beg.v.int_val, end.v.int_val));
 
-    if (end.v.int_val > strlen(full.v.string_val))
-	end.v.int_val = strlen(full.v.string_val);
+    if (end.v.int_val > gp_strlen(full.v.string_val))
+	end.v.int_val = gp_strlen(full.v.string_val);
     if (beg.v.int_val < 1)
 	beg.v.int_val = 1;
-    if (beg.v.int_val > end.v.int_val)
-	beg.v.int_val = strlen(full.v.string_val)+1;
 
-    full.v.string_val[end.v.int_val] = '\0';
-    push(Gstring(&substr, &full.v.string_val[beg.v.int_val-1]));
+    if (beg.v.int_val > end.v.int_val) {
+	push(Gstring(&substr, ""));
+    } else {
+	char *begp = gp_strchrn(full.v.string_val,beg.v.int_val-1);
+	char *endp = gp_strchrn(full.v.string_val,end.v.int_val);
+	*endp = '\0';
+	push(Gstring(&substr, begp));
+    }
     gpfree_string(&full);
 }
 
@@ -1164,6 +1222,7 @@ f_sprintf(union argument *arg)
     int prev_pos;
     int i, remaining;
     int nargs = 0;
+    int save_errno;
     enum DATA_TYPES spec_type;
 
     /* Retrieve number of parameters from top of stack */
@@ -1232,6 +1291,7 @@ f_sprintf(union argument *arg)
 
 #ifdef HAVE_SNPRINTF
 	/* Use the format to print next arg */
+	save_errno = errno;
 	switch(spec_type) {
 	case INTGR:
 	    snprintf(outpos,bufsize-(outpos-buffer),
@@ -1248,6 +1308,11 @@ f_sprintf(union argument *arg)
 	default:
 	    int_error(NO_CARET,"internal error: invalid spec_type");
 	}
+#if _MSC_VER
+       buffer[bufsize-1] = '\0';	/* VC++ is not ANSI-compliant */
+       if (errno == ERANGE) errno = save_errno;
+#endif 
+
 #else
 	/* FIXME - this is bad; we should dummy up an snprintf equivalent */
 	switch(spec_type) {
@@ -1322,7 +1387,7 @@ f_gprintf(union argument *arg)
     struct value fmt, val, result;
     char *buffer;
     int length;
-    double base;
+    double base = 10.;
  
     /* Retrieve parameters from top of stack */
     pop(&val);
@@ -1340,9 +1405,6 @@ f_gprintf(union argument *arg)
     if (fmt.type != STRING)
 	int_error(NO_CARET,"First parameter to gprintf must be a format string");
 
-    /* EAM FIXME - I have no idea where we would learn another base is wanted */
-    base = 10.;
-
     /* Make sure we have at least as much space in the output as the format itself */
     length = 80 + strlen(fmt.v.string_val);
     buffer = gp_alloc(length, "f_gprintf");
@@ -1410,6 +1472,7 @@ f_strptime(union argument *arg)
 {
     struct value fmt, val;
     struct tm time_tm;
+    double usec = 0.0;
     double result;
 
     (void) arg; /* Avoid compiler warnings */
@@ -1424,18 +1487,67 @@ f_strptime(union argument *arg)
 	int_error(NO_CARET, "Internal error: string not allocated");
 
 
-    /* string -> time_tm */
-    gstrptime(val.v.string_val, fmt.v.string_val, &time_tm);
+    /* string -> time_tm  plus extra fractional second */
+    gstrptime(val.v.string_val, fmt.v.string_val, &time_tm, &usec);
 
     /* time_tm -> result */
     result = gtimegm(&time_tm);
     FPRINTF((stderr," strptime result = %g seconds \n", result));
 
+    /* Add back any extra fractional second */
+    result += usec;
+
     gpfree_string(&val);
     gpfree_string(&fmt);
     push(Gcomplex(&val, result, 0.0));
 }
 
+/* Get current system time in seconds since 2000 
+ * The type of the value popped from the stack 
+ * determines what is returned.
+ * If integer, the result is also an integer.
+ * If real (complex), the result is also real, 
+ * with microsecond precision (if available).
+ * If string, it is assumed to be a format string, 
+ * and it is passed to strftime to get a formatted time string.
+ */
+void
+f_time(union argument *arg)
+{
+    struct value val, val2;
+    double time_now;
+#ifdef HAVE_SYS_TIME_H
+    struct timeval tp;
+
+    gettimeofday(&tp, NULL);
+    tp.tv_sec -= SEC_OFFS_SYS;
+    time_now = tp.tv_sec + (tp.tv_usec/1000000.0);
+#else
+
+    time_now = (double) time(NULL);
+    time_now -= SEC_OFFS_SYS;
+#endif
+
+    (void) arg; /* Avoid compiler warnings */
+    pop(&val); 
+    
+    switch(val.type) {
+	case INTGR:
+	    push(Ginteger(&val, (int) time_now));
+	    break;
+	case CMPLX:
+	    push(Gcomplex(&val, time_now, 0.0));
+	    break;
+	case STRING:
+	    push(&val); /* format string */
+	    push(Gcomplex(&val2, time_now, 0.0));
+	    f_strftime(arg);
+	    break;
+	default:
+	    int_error(NO_CARET,"internal error: invalid argument type");
+    }
+}
+
 
 /* Return which argument type sprintf will need for this format string:
  *   char*       STRING
