@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: plot.c,v 1.128.2.10 2012/03/02 20:00:04 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: plot.c,v 1.128.2.16 2013/04/05 16:39:50 markisch Exp $"); }
 #endif
 
 /* GNUPLOT - plot.c */
@@ -135,7 +135,8 @@ static void wrapper_for_write_history __PROTO((void));
 #endif				/* GNUPLOT_HISTORY */
 
 TBOOLEAN interactive = TRUE;	/* FALSE if stdin not a terminal */
-TBOOLEAN noinputfiles = TRUE; /* FALSE if there are script files */
+TBOOLEAN noinputfiles = TRUE;	/* FALSE if there are script files */
+TBOOLEAN skip_gnuplotrc = FALSE;/* skip system gnuplotrc and ~/.gnuplot */
 TBOOLEAN persist_cl = FALSE; /* TRUE if -persist is parsed in the command line */
 
 /* user home directory */
@@ -328,12 +329,16 @@ main(int argc, char **argv)
 
 #if defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)
     /* T.Walter 1999-06-24: 'rl_readline_name' must be this fix name.
-     * It is used to parse a 'gnuplot' specific section in '~/.inputrc' 
+     * It is used to parse a 'gnuplot' specific section in '~/.inputrc'
      * or gnuplot specific commands in '.editrc' (when using editline
      * instead of readline) */
     rl_readline_name = "Gnuplot";
     rl_terminal_name = getenv("TERM");
+#if defined(HAVE_LIBREADLINE)
     using_history();
+#else
+    history_init();
+#endif
 #endif
 #if defined(HAVE_LIBREADLINE) && !defined(MISSING_RL_TILDE_EXPANSION)
     rl_complete_with_tilde_expansion = 1;
@@ -356,6 +361,7 @@ main(int argc, char **argv)
 		    "  -V, --version\n"
 		    "  -h, --help\n"
 		    "  -p  --persist\n"
+		    "  -d  --default-settings\n"
 		    "  -e  \"command1; command2; ...\"\n"
 		    "gnuplot %s patchlevel %s\n",
 		    gnuplot_version, gnuplot_patchlevel);
@@ -371,8 +377,15 @@ main(int argc, char **argv)
 #endif
 	    return 0;
 
-	} else if (!strncmp(argv[i], "-persist", 2) || !strcmp(argv[i], "--persist")) {
+	} else if (!strncmp(argv[i], "-persist", 2) || !strcmp(argv[i], "--persist")
+#ifdef _Windows
+		|| !stricmp(argv[i], "-noend") || !stricmp(argv[i], "/noend")
+#endif
+		) {
 	    persist_cl = TRUE;
+	} else if (!strncmp(argv[i], "-d", 2) || !strcmp(argv[i], "--default-settings")) {
+	    /* Skip local customization read from ~/.gnuplot */
+	    skip_gnuplotrc = TRUE;
 	}
     }
 
@@ -388,30 +401,20 @@ main(int argc, char **argv)
     setbuf(stderr, (char *) NULL);
 
 #ifdef HAVE_SETVBUF
-    /* this was once setlinebuf(). Docs say this is
+    /* This was once setlinebuf(). Docs say this is
      * identical to setvbuf(,NULL,_IOLBF,0), but MS C
      * faults this (size out of range), so we try with
      * size of 1024 instead. [SAS/C does that, too. -lh]
-     * Failing this, I propose we just make the call and
-     * ignore the return : its probably not a big deal
      */
     if (setvbuf(stdout, (char *) NULL, _IOLBF, (size_t) 1024) != 0)
 	(void) fputs("Could not linebuffer stdout\n", stderr);
 
-#ifdef X11
-    /* This call used to be in x11.trm, with the following comment:
-     *   Multi-character inputs like escape sequences but also mouse-pasted
-     *   text got buffered and therefore didn't trigger the select() function
-     *   in X11_waitforinput(). Switching to unbuffered input solved this.
-     *   23 Jan 2002 (joze)
-     * But switching to unbuffered mode causes all characters in the input
+    /* Switching to unbuffered mode causes all characters in the input
      * buffer to be lost. So the only safe time to do it is on program entry.
-     * The #ifdef X11 is probably unnecessary, but makes the change minimal.
-     * Do any non-X platforms suffer from the same problem?
-     * EAM - Jan 2004.
+     * Do any non-X platforms suffer from this problem?
+     * EAM - Jan 2013 YES.
      */
     setvbuf(stdin, (char *) NULL, _IONBF, 0);
-#endif
 
 #endif
 
@@ -440,10 +443,22 @@ main(int argc, char **argv)
     interactive = isatty(fileno(stdin));
 # endif
 
-    if (argc > 1)
-	interactive = noinputfiles = FALSE;
-    else
-	noinputfiles = TRUE;
+    /* Note: we want to know whether this is an interactive session so that we can
+     * decide whether or not to write status information to stderr.  The old test
+     * for this was to see if (argc > 1) but the addition of optional command line
+     * switches broke this.  What we really wanted to know was whether any of the
+     * command line arguments are file names or an explicit in-line "-e command".
+     */
+    for (i = 1; i < argc; i++) {
+# ifdef _Windows
+	if (!stricmp(argv[i], "/noend"))
+	    continue;
+# endif
+	if ((argv[i][0] != '-') || (argv[i][1] == 'e')) {
+	    interactive = FALSE;
+	    break;
+	}
+    }
 
     /* Need this before show_version is called for the first time */
 
@@ -496,7 +511,7 @@ main(int argc, char **argv)
 	fprintf(stderr,
 	    "\ngnuplot changed the codepage of this console from %i to %i to\n" \
 	    "match the graph window. Some characters might only display correctly\n" \
-	    "if you change the font to a non-raster type.\n", 
+	    "if you change the font to a non-raster type.\n",
 	    cp_input, GetConsoleCP());
     }
 #else
@@ -504,7 +519,7 @@ main(int argc, char **argv)
 	fprintf(stderr,
 	    "\nWarning: The codepage of the graph window (%i) and that of the\n" \
 	    "console (%i) differ. Use `set encoding` or `!chcp` if extended\n" \
-	    "characters don't display correctly.\n", 
+	    "characters don't display correctly.\n",
 	    GetACP(), GetConsoleCP());
     }
 #endif
@@ -615,25 +630,20 @@ main(int argc, char **argv)
 	}
     }
 
-    if (argc > 1) {
-#ifdef _Windows
-	TBOOLEAN noend = persist_cl;
-#endif
-
-	/* load filenames given as arguments */
-	while (--argc > 0) {
+    /* load filenames given as arguments */
+    while (--argc > 0) {
 	    ++argv;
 	    c_token = 0;
+	    if (!strncmp(*argv, "-persist", 2) || !strcmp(*argv, "--persist")
 #ifdef _Windows
-	    if (stricmp(*argv, "-noend") == 0 || stricmp(*argv, "/noend") == 0
-	       	|| stricmp(*argv, "-persist") == 0)
-		noend = TRUE;
-	    else
+		|| !stricmp(*argv, "-noend") || !stricmp(*argv, "/noend")
 #endif
-	    if (!strncmp(*argv, "-persist", 2) || !strcmp(*argv, "--persist")) {
+	    ) {
 		FPRINTF((stderr,"'persist' command line option recognized\n"));
-
 	    } else if (strcmp(*argv, "-") == 0) {
+#if defined(_Windows) && !defined(WGP_CONSOLE)
+		TextShow(&textwin);
+#endif
 		interactive = TRUE;
 		while (!com_line());
 		interactive = FALSE;
@@ -644,21 +654,34 @@ main(int argc, char **argv)
 		    fprintf(stderr, "syntax:  gnuplot -e \"commands\"\n");
 		    return 0;
 		}
+		interactive = FALSE;
+		noinputfiles = FALSE;
 		do_string(*argv);
 
+	    } else if (!strncmp(*argv, "-d", 2) || !strcmp(*argv, "--default-settings")) {
+		/* Ignore this; it already had its effect */
+		FPRINTF((stderr, "ignoring -d\n"));
+	    } else if (*argv[0] == '-') {
+		fprintf(stderr, "unrecognized option %s\n", *argv);
 	    } else {
+		interactive = FALSE;
+		noinputfiles = FALSE;
 		load_file(loadpath_fopen(*argv, "r"), gp_strdup(*argv), FALSE);
 	    }
-	}
+    }
+
 #ifdef _Windows
-	if (noend) {
-	    interactive = TRUE;
-	    while (!com_line());
-	}
-#endif
-    } else {
-	/* take commands from stdin */
+    /* On Windows 'persist' is handled by keeping the main input loop running. */
+    if (persist_cl) {
+	interactive = TRUE;
 	while (!com_line());
+	interactive = FALSE;
+    } else
+#endif
+    {
+	/* take commands from stdin */
+	if (noinputfiles)
+	    while (!com_line());
     }
 
 #if (defined(HAVE_LIBREADLINE) || defined(HAVE_LIBEDITLINE)) && defined(GNUPLOT_HISTORY)
@@ -711,13 +734,16 @@ load_rcfile(int where)
     FILE *plotrc = NULL;
     char *rcfile = NULL;
 
+    if (skip_gnuplotrc)
+	return;
+
     if (where == 0) {
 #ifdef GNUPLOT_SHARE_DIR
 # if defined(_Windows)
 	/* retrieve path relative to gnuplot executable,
 	 * whose path is in szModuleName (winmain.c) */
-	rcfile = gp_alloc(strlen((char *)szPackageDir) + 1 +
-		strlen(GNUPLOT_SHARE_DIR) + 1 + strlen("gnuplotrc") + 1, "rcfile");
+	rcfile = gp_alloc(strlen((char *)szPackageDir) + 1
+	       + strlen(GNUPLOT_SHARE_DIR) + 1 + strlen("gnuplotrc") + 1, "rcfile");
 	strcpy(rcfile, (char *)szPackageDir);
 	PATH_CONCAT(rcfile, GNUPLOT_SHARE_DIR);
 # else
