@@ -56,7 +56,7 @@ QtGnuplotScene::QtGnuplotScene(QtGnuplotEventHandler* eventHandler, QObject* par
 	m_eventHandler = eventHandler;
 	m_lastModifierMask = 0;
 	m_textAngle = 0.;
-	m_textAlignment == Qt::AlignLeft;
+	m_textAlignment = Qt::AlignLeft;
 	m_currentZ = 1.;
 	m_currentPointSize = 1.;
 	m_enhanced = 0;
@@ -347,6 +347,7 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 			m_zoomStopText->setPlainText(text); /// @todo font
 			m_zoomStopText->setPos(m_lastMousePos);
 			m_zoomRect->setRect(QRectF(m_zoomBoxCorner + QPointF(0.5, 0.5), m_lastMousePos + QPointF(0.5, 0.5)).normalized());
+			m_zoomRect->setZValue(32767);  // make sure guide box is on top
 		}
 	}
 	else if (type == GELineTo)
@@ -379,7 +380,7 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 			newgroup = createItemGroup(m_currentGroup);
 			newgroup->setZValue(m_currentZ++);
 			// Copy the visible/hidden status from the previous plot/replot
-			if (0 < m_currentPlotNumber  && m_currentPlotNumber < m_key_boxes.count())
+			if (0 < m_currentPlotNumber && m_currentPlotNumber <= m_key_boxes.count())
 				newgroup->setVisible( !(m_key_boxes[m_currentPlotNumber-1].ishidden()) );
 			// Store it in an ordered list so we can toggle it by index
 			if (m_currentPlotNumber >= m_plot_group.count())
@@ -540,7 +541,9 @@ void QtGnuplotScene::mousePressEvent(QGraphicsSceneMouseEvent* event)
 	else if (event->button()== Qt::MidButton)   button = 2;
 	else if (event->button()== Qt::RightButton) button = 3;
 
-	m_eventHandler->postTermEvent(GE_buttonpress, int(event->scenePos().x()), int(event->scenePos().y()), button, 0, 0); /// @todo m_id
+	m_eventHandler->postTermEvent(GE_buttonpress, 
+			int(event->scenePos().x()), int(event->scenePos().y()), 
+			button, 0, 0); /// @todo m_id
 	QGraphicsScene::mousePressEvent(event);
 }
 
@@ -551,11 +554,18 @@ void QtGnuplotScene::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
 	{
 		m_lineTo->hide();
 		QString status;
-		for (int i = 0; i < 4; i++)
-			if (m_axisValid[i])
-				status += (i % 2 ? QString("y") : QString("x")) + (i / 2 ? QString("2"): QString("")) + " = "
-				       + QString::number(sceneToGraph(0, i % 2 ? event->scenePos().y() : event->scenePos().x()));
-		//status << sceneToGraph(0, event->scenePos().x()) << sceneToGraph(1, event->scenePos().y());
+		if (m_axisValid[0])
+			status += QString("x = ")
+			+ QString::number(sceneToGraph(0,event->scenePos().x()));
+		if (m_axisValid[1])
+			status += QString(" y = ")
+			+ QString::number(sceneToGraph(1,event->scenePos().y()));
+		if (m_axisValid[2])
+			status += QString(" x2 = ")
+			+ QString::number(sceneToGraph(2,event->scenePos().x()));
+		if (m_axisValid[3])
+			status += QString(" y2 = ")
+			+ QString::number(sceneToGraph(3,event->scenePos().y()));
 		m_widget->setStatusText(status);
 		QGraphicsScene::mouseMoveEvent(event);
 		return;
@@ -620,11 +630,13 @@ void QtGnuplotScene::wheelEvent(QGraphicsSceneWheelEvent* event)
 	updateModifiers();
 	if (event->orientation() == Qt::Horizontal) {
 		// 6 = scroll left, 7 = scroll right
-		m_eventHandler->postTermEvent(GE_buttonpress, 0, 0, 
+		m_eventHandler->postTermEvent(GE_buttonpress,
+			int(event->scenePos().x()), int(event->scenePos().y()), 
 			event->delta() > 0 ? 6 : 7, 0, 0);
 	} else { /* if (event->orientation() == Qt::Vertical) */
 		// 4 = scroll up, 5 = scroll down
-		m_eventHandler->postTermEvent(GE_buttonpress, 0, 0, 
+		m_eventHandler->postTermEvent(GE_buttonpress,
+			int(event->scenePos().x()), int(event->scenePos().y()), 
 			event->delta() > 0 ? 4 : 5, 0, 0);
 	} 
 }
