@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: unset.c,v 1.128.2.4 2010/12/05 00:01:23 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: unset.c,v 1.149.2.2 2012/10/09 16:30:15 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - unset.c */
@@ -41,6 +41,7 @@ static char *RCSid() { return RCSid("$Id: unset.c,v 1.128.2.4 2010/12/05 00:01:2
 #include "contour.h"
 #include "datafile.h"
 #include "fit.h"
+#include "gadgets.h"
 #include "gp_hist.h"
 #include "hidden3d.h"
 #include "misc.h"
@@ -62,7 +63,7 @@ static void delete_arrow __PROTO((struct arrow_def *, struct arrow_def *));
 static void unset_autoscale __PROTO((void));
 static void unset_bars __PROTO((void));
 static void unset_border __PROTO((void));
-
+static void unset_boxplot __PROTO((void));
 static void unset_boxwidth __PROTO((void));
 static void unset_fillstyle __PROTO((void));
 static void unset_clabel __PROTO((void));
@@ -84,11 +85,14 @@ static void unset_key __PROTO((void));
 static void unset_keytitle __PROTO((void));
 static void unset_label __PROTO((void));
 static void delete_label __PROTO((struct text_label * prev, struct text_label * this));
+static void unset_linestyle __PROTO((struct linestyle_def **head));
+static void unset_linetype __PROTO((void));
 #ifdef EAM_OBJECTS
 static void unset_object __PROTO((void));
 static void delete_object __PROTO((struct object * prev, struct object * this));
 static void unset_style_rectangle __PROTO(());
 static void unset_style_circle __PROTO(());
+static void unset_style_ellipse __PROTO(());
 #endif
 static void unset_loadpath __PROTO((void));
 static void unset_locale __PROTO((void));
@@ -119,6 +123,7 @@ static void unset_pointsize __PROTO((void));
 static void unset_pointintervalbox __PROTO((void));
 static void unset_polar __PROTO((void));
 static void unset_print __PROTO((void));
+static void unset_psdir __PROTO((void));
 static void unset_samples __PROTO((void));
 static void unset_size __PROTO((void));
 static void unset_style __PROTO((void));
@@ -149,7 +154,7 @@ unset_command()
 
     c_token++;
 
-    check_for_iteration();
+    set_iterator = check_for_iteration();
 
     found_token = lookup_table(&set_tbl[0],c_token);
 
@@ -230,6 +235,9 @@ unset_command()
     case S_LABEL:
 	unset_label();
 	break;
+    case S_LINETYPE:
+	unset_linetype();
+	break;
     case S_LOADPATH:
 	unset_loadpath();
 	break;
@@ -278,7 +286,6 @@ unset_command()
 	break;
 #endif
     case S_MULTIPLOT:
-/*	unset_multiplot(); */
 	term_end_multiplot();
 	break;
     case S_OFFSETS:
@@ -314,11 +321,17 @@ unset_command()
     case S_PRINT:
 	unset_print();
 	break;
+    case S_PSDIR:
+	unset_psdir();
+	break;
 #ifdef EAM_OBJECTS
     case S_OBJECT:
 	unset_object();
 	break;
 #endif
+    case S_RTICS:
+	unset_tics(POLAR_AXIS);
+	break;
     case S_SAMPLES:
 	unset_samples();
 	break;
@@ -425,16 +438,9 @@ unset_command()
 	break;
     case S_XDATA:
 	unset_timedata(FIRST_X_AXIS);
-	/* FIXME HBB 20000506: does unsetting these axes make *any*
-	 * sense?  After all, their content is never displayed, so
-	 * what would they need a corrected format for? */
-	unset_timedata(T_AXIS);
-	unset_timedata(U_AXIS);
 	break;
     case S_YDATA:
 	unset_timedata(FIRST_Y_AXIS);
-	/* FIXME: see above */
-	unset_timedata(V_AXIS);
 	break;
     case S_ZDATA:
 	unset_timedata(FIRST_Z_AXIS);
@@ -485,7 +491,7 @@ unset_command()
 	unset_range(COLOR_AXIS);
 	break;
     case S_RRANGE:
-	unset_range(R_AXIS);
+	unset_range(POLAR_AXIS);
 	break;
     case S_TRANGE:
 	unset_range(T_AXIS);
@@ -496,6 +502,10 @@ unset_command()
     case S_VRANGE:
 	unset_range(V_AXIS);
 	break;
+    case S_RAXIS:
+	raxis = FALSE;
+	c_token++;
+	break;
     case S_XZEROAXIS:
 	unset_zeroaxis(FIRST_X_AXIS);
 	break;
@@ -520,13 +530,14 @@ unset_command()
 	break;
     }
 
-    if (next_iteration()) {
+    if (next_iteration(set_iterator)) {
 	c_token = save_token;
 	goto ITERATE;
     }
 
-    /* FIXME - Should this be inside the iteration loop? */
     update_gpval_variables(0);
+    
+    set_iterator = cleanup_iteration(set_iterator);
 }
 
 
@@ -608,7 +619,9 @@ static void
 unset_autoscale()
 {
     if (END_OF_COMMAND) {
-	INIT_AXIS_ARRAY(set_autoscale, FALSE);
+	int axis;
+	for (axis=0; axis<AXIS_ARRAY_SIZE; axis++)
+	    axis_array[axis].set_autoscale = FALSE;
     } else if (equals(c_token, "xy") || equals(c_token, "tyx")) {
 	axis_array[FIRST_X_AXIS].set_autoscale
 	    = axis_array[FIRST_Y_AXIS].set_autoscale = AUTOSCALE_NONE;
@@ -643,6 +656,15 @@ unset_border()
 }
 
 
+/* process 'unset style boxplot' command */
+static void
+unset_boxplot()
+{
+    boxplot_style defstyle = DEFAULT_BOXPLOT_STYLE;
+    boxplot_opts = defstyle;
+}
+
+
 /* process 'unset boxwidth' command */
 static void
 unset_boxwidth()
@@ -795,6 +817,7 @@ unset_format()
 	SET_DEFFORMAT(SECOND_X_AXIS, set_for_axis);
 	SET_DEFFORMAT(SECOND_Y_AXIS, set_for_axis);
 	SET_DEFFORMAT(COLOR_AXIS   , set_for_axis);
+	SET_DEFFORMAT(POLAR_AXIS   , set_for_axis);
     }
 }
 
@@ -820,11 +843,7 @@ unset_grid()
 static void
 unset_hidden3d()
 {
-#ifdef LITE
-    printf(" Hidden Line Removal Not Supported in LITE version\n");
-#else
     hidden3d = FALSE;
-#endif
 }
 
 static void
@@ -936,6 +955,31 @@ delete_label(struct text_label *prev, struct text_label *this)
     }
 }
 
+static void
+unset_linestyle(struct linestyle_def **head)
+{
+    int tag = int_expression();
+    struct linestyle_def *this, *prev;
+    for (this = *head, prev = NULL; this != NULL; 
+	 prev = this, this = this->next) {
+	if (this->tag == tag) {
+	    delete_linestyle(head, prev, this);
+	    break;
+	}
+    }
+}
+
+static void
+unset_linetype()
+{
+    if (equals(c_token,"cycle")) {
+	linetype_recycle_count = 0;
+	c_token++;
+    }
+    else if (!END_OF_COMMAND)
+	unset_linestyle(&first_perm_linestyle);
+}
+
 #ifdef EAM_OBJECTS
 /* process 'unset rectangle' command */
 static void
@@ -962,7 +1006,9 @@ unset_object()
 		return;		/* exit, our job is done */
 	    }
 	}
-	int_error(c_token, "object not found");
+	/*
+	int_warn(c_token, "object %d not found", tag);
+	 */
     }
 }
 
@@ -1008,8 +1054,11 @@ unset_locale()
 static void
 reset_logscale(AXIS_INDEX axis)
 {
+    TBOOLEAN undo_rlog = (axis == POLAR_AXIS && R_AXIS.log);
     axis_array[axis].log = FALSE;
     axis_array[axis].base = 0.0;
+    if (undo_rlog)
+	rrange_to_xy();
 }
 
 /* process 'unset logscale' command */
@@ -1029,11 +1078,11 @@ unset_logscale()
 	/* do reverse search because of "x", "x1", "x2" sequence in
 	 * axisname_tbl */
 	while (i < token[c_token].length) {
-	    axis = lookup_table_nth_reverse(axisname_tbl, AXIS_ARRAY_SIZE,
+	    axis = lookup_table_nth_reverse(axisname_tbl, LAST_REAL_AXIS+1,
 					    gp_input_line + token[c_token].start_index + i);
 	    if (axis < 0) {
 		token[c_token].start_index += i;
-		int_error(c_token, "unknown axis");
+		int_error(c_token, "invalid axis");
 	    }
 	    reset_logscale(axisname_tbl[axis].value);
 	    i += strlen(axisname_tbl[axis].key);
@@ -1133,6 +1182,7 @@ unset_tics(AXIS_INDEX axis)
 	axis_array[i].ticscale = 1.0;
 	axis_array[i].miniticscale = 0.5;
 	axis_array[i].tic_in = TRUE;
+	axis_array[i].manual_justify = FALSE;
 
 	free_marklist(axis_array[i].ticdef.def.user);
 	axis_array[i].ticdef.def.user = NULL;
@@ -1187,6 +1237,13 @@ unset_print()
     print_set_output(NULL, FALSE);
 }
 
+/* process 'unset psdir' command */
+static void
+unset_psdir()
+{
+    free(PS_psdir);
+    PS_psdir = NULL;
+}
 
 /* process 'unset parametric' command */
 static void
@@ -1315,8 +1372,10 @@ unset_style()
 #ifdef EAM_OBJECTS
 	unset_style_rectangle();
 	unset_style_circle();
+	unset_style_ellipse();
 #endif
 	unset_histogram();
+	unset_boxplot();
 	c_token++;
 	return;
     }
@@ -1336,15 +1395,7 @@ unset_style()
 	    while (first_linestyle != NULL)
 		delete_linestyle(&first_linestyle, NULL, first_linestyle);
 	} else {
-	    int tag = int_expression();
-	    struct linestyle_def *this, *prev;
-	    for (this = first_linestyle, prev = NULL; this != NULL; 
-		 prev = this, this = this->next) {
-		if (this->tag == tag) {
-		    delete_linestyle(&first_linestyle, prev, this);
-		    break;
-		}
-	    }
+	    unset_linestyle(&first_linestyle);
 	}
 	break;
     case SHOW_STYLE_FILLING:
@@ -1362,10 +1413,21 @@ unset_style()
 #ifdef EAM_OBJECTS
     case SHOW_STYLE_RECTANGLE:
 	unset_style_rectangle();
+	c_token++;
+	break;
+    case SHOW_STYLE_CIRCLE:
 	unset_style_circle();
 	c_token++;
 	break;
+    case SHOW_STYLE_ELLIPSE:
+	unset_style_ellipse();
+	c_token++;
+	break;
 #endif
+    case SHOW_STYLE_BOXPLOT:
+	unset_boxplot();
+	c_token++;
+	break;
     default:
 	int_error(c_token, "expecting 'data', 'function', 'line', 'fill' or 'arrow'");
     }
@@ -1392,15 +1454,21 @@ unset_table()
 
 
 /* process 'unset terminal' comamnd */
+/* Aug 2012:  restore original terminal type */
 static void
 unset_terminal()
 {
-    /* This is a problematic case */
-/* FIXME */
-    if (multiplot)
-	int_error(c_token, "You can't change the terminal in multiplot mode");
+    struct udvt_entry *original_terminal = get_udv_by_name("GNUTERM");
 
-    list_terms();
+    if (multiplot)
+	term_end_multiplot();
+
+    term_reset();
+
+    if (original_terminal) {
+	char *termname = original_terminal->udv_value.v.string_val;
+	term = change_term(termname, strlen(termname));
+    }
     screen_ok = FALSE;
 }
 
@@ -1470,6 +1538,7 @@ unset_view()
     surface_rot_z = 30.0;
     surface_rot_x = 60.0;
     surface_scale = 1.0;
+    surface_lscale = 0.0;
     surface_zscale = 1.0;
 }
 
@@ -1485,7 +1554,7 @@ unset_zero()
 static void
 unset_timedata(AXIS_INDEX axis)
 {
-    axis_array[axis].is_timedata = FALSE;
+    axis_array[axis].datatype = DT_NORMAL;
 }
 
 
@@ -1556,8 +1625,7 @@ reset_command()
 {
     AXIS_INDEX axis;
     TBOOLEAN save_interactive = interactive;
-    static const TBOOLEAN set_for_axis[AXIS_ARRAY_SIZE]
-	= AXIS_ARRAY_INITIALIZER(TRUE);
+    static TBOOLEAN set_for_axis[AXIS_ARRAY_SIZE] = AXIS_ARRAY_INITIALIZER(TRUE);
 
     c_token++;
 
@@ -1601,6 +1669,7 @@ reset_command()
 	delete_object((struct object *) NULL, first_object);
     unset_style_rectangle();
     unset_style_circle();
+    unset_style_ellipse();
 #endif
 
     /* 'polar', 'parametric' and 'dummy' are interdependent, so be
@@ -1630,6 +1699,9 @@ reset_command()
 	axis_array[axis].writeback_max = axis_array[axis].set_max
 	    = axis_defaults[axis].max;
 
+	axis_array[axis].min_constraint = CONSTRAINT_NONE;
+	axis_array[axis].max_constraint = CONSTRAINT_NONE;
+	
 	/* 'tics' default is on for some, off for the other axes: */
 	unset_tics(axis);
 	axis_array[axis].ticmode = axis_defaults[axis].ticmode;
@@ -1639,7 +1711,9 @@ reset_command()
 
 	reset_logscale(axis);
     }
+    raxis = TRUE;
 
+    unset_boxplot();
     unset_boxwidth();
 
     clip_points = FALSE;
@@ -1732,4 +1806,11 @@ unset_style_circle()
     default_circle = foo;
     return;
 }
+static void
+unset_style_ellipse()
+{
+    struct object foo = DEFAULT_ELLIPSE_STYLE;
+    default_ellipse = foo;
+    return;
+}
 #endif
