@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: util.c,v 1.99 2011/11/10 05:15:58 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: util.c,v 1.99.2.6 2013/02/08 23:01:39 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - util.c */
@@ -499,16 +499,8 @@ mant_exp(
 
 /*}}} */
 
-/*
- * Kludge alert!!
- * Workaround until we have a better solution ...
- * Note: this assumes that all calls to sprintf in gprintf have
- * exactly three args. Lars
- */
-#ifdef HAVE_SNPRINTF
-# define sprintf(str,fmt,arg) \
-    if (snprintf((str),count,(fmt),(arg)) > count) \
-      fprintf (stderr,"%s:%d: Warning: too many digits for format\n",__FILE__,__LINE__)
+#ifndef HAVE_SNPRINTF
+# define snprintf(str,len,fmt,arg) sprintf(str,fmt,arg)
 #endif
 
 /*{{{  gprintf */
@@ -519,14 +511,20 @@ mant_exp(
  * '1.0*10^1'.  This causes problems for people using the %T part,
  * only, with logscaled axes, in combination with the occasional
  * round-off error. */
+/* EAM Nov 2012:
+ * Unbelievably, the count parameter has been silently ignored or
+ * improperly applied ever since this routine was introduced back
+ * in version 3.7.  Now fixed to prevent buffer overflow.
+ */
 void
 gprintf(
-    char *dest,
+    char *outstring,
     size_t count,
     char *format,
     double log10_base,
     double x)
 {
+    char tempdest[MAX_LINE_LEN + 1];
     char temp[MAX_LINE_LEN + 1];
     char *t;
     TBOOLEAN seen_mantissa = FALSE; /* remember if mantissa was already output */
@@ -534,12 +532,19 @@ gprintf(
     int stored_power = 0;	/* power matching the mantissa output earlier */
     TBOOLEAN got_hash = FALSE;				   
 
+    char *dest = &tempdest[0];
+    char *limit = &tempdest[MAX_LINE_LEN];
+#define remaining_space (size_t)(limit-dest)
+
+    *dest = '\0';
+
     set_numeric_locale();
 
     for (;;) {
 	/*{{{  copy to dest until % */
 	while (*format != '%')
-	    if (!(*dest++ = *format++)) {
+	    if (!(*dest++ = *format++) || (remaining_space == 0)) {
+		safe_strncpy(outstring,tempdest,count);
 		reset_numeric_locale();
 		return;		/* end of format */
 	    }
@@ -580,11 +585,11 @@ gprintf(
 		t[1] = 'l';
 		t[2] = *format;
 		t[3] = '\0';
-		sprintf(dest, temp, (long long) x);
+		snprintf(dest, remaining_space, temp, (long long) x);
 	    } else {
 		t[0] = *format;
 		t[1] = '\0';
-		sprintf(dest, temp, (int) x);
+		snprintf(dest, remaining_space, temp, (int) x);
 	    }
 	    break;
 	    /*}}} */
@@ -597,7 +602,7 @@ gprintf(
 	case 'G':
 	    t[0] = *format;
 	    t[1] = 0;
-	    sprintf(dest, temp, x);
+	    snprintf(dest, remaining_space, temp, x);
 	    break;
 	    /*}}} */
 	    /*{{{  l --- mantissa to current log base */
@@ -611,7 +616,7 @@ gprintf(
 		mant_exp(stored_power_base, x, FALSE, &mantissa,
 				&stored_power, temp);
 		seen_mantissa = TRUE;
-		sprintf(dest, temp, mantissa);
+		snprintf(dest, remaining_space, temp, mantissa);
 		break;
 	    }
 	    /*}}} */
@@ -626,7 +631,7 @@ gprintf(
 		mant_exp(stored_power_base, x, FALSE, &mantissa,
 				&stored_power, temp);
 		seen_mantissa = TRUE;
-		sprintf(dest, temp, mantissa);
+		snprintf(dest, remaining_space, temp, mantissa);
 		break;
 	    }
 	    /*}}} */
@@ -641,7 +646,7 @@ gprintf(
 		mant_exp(stored_power_base, x, TRUE, &mantissa,
 				&stored_power, temp);
 		seen_mantissa = TRUE;
-		sprintf(dest, temp, mantissa);
+		snprintf(dest, remaining_space, temp, mantissa);
 		break;
 	    }
 	    /*}}} */
@@ -656,7 +661,7 @@ gprintf(
 		mant_exp(stored_power_base, x, FALSE, &mantissa,
 				&stored_power, temp);
 		seen_mantissa = TRUE;
-		sprintf(dest, temp, mantissa);
+		snprintf(dest, remaining_space, temp, mantissa);
 		break;
 	    }
 	    /*}}} */
@@ -675,7 +680,7 @@ gprintf(
 		else
 		    stored_power_base = log10_base;
 		    mant_exp(log10_base, x, FALSE, NULL, &power, "%.0f");
-		sprintf(dest, temp, power);
+		snprintf(dest, remaining_space, temp, power);
 		break;
 	    }
 	    /*}}} */
@@ -693,7 +698,7 @@ gprintf(
 			int_error(NO_CARET, "Format character mismatch: %%T is only valid with %%t");
 		else
 		    mant_exp(1.0, x, FALSE, NULL, &power, "%.0f");
-		sprintf(dest, temp, power);
+		snprintf(dest, remaining_space, temp, power);
 		break;
 	    }
 	    /*}}} */
@@ -711,7 +716,7 @@ gprintf(
 			int_error(NO_CARET, "Format character mismatch: %%S is only valid with %%s");
 		else
 		    mant_exp(1.0, x, TRUE, NULL, &power, "%.0f");
-		sprintf(dest, temp, power);
+		snprintf(dest, remaining_space, temp, power);
 		break;
 	    }
 	    /*}}} */
@@ -734,7 +739,7 @@ gprintf(
 		    /* -18 -> 0, 0 -> 6, +18 -> 12, ... */
 		    /* HBB 20010121: avoid division of -ve ints! */
 		    power = (power + 24) / 3;
-		    sprintf(dest, temp, "yzafpnum kMGTPEZY"[power]);
+		    snprintf(dest, remaining_space, temp, "yzafpnum kMGTPEZY"[power]);
 		} else {
 		    /* please extend the range ! */
 		    /* name  power   name  power
@@ -749,7 +754,7 @@ gprintf(
 		       milli   -3    kilo    3   */
 
 		    /* fall back to simple exponential */
-		    sprintf(dest, "e%+02d", power);
+		    snprintf(dest, remaining_space, "e%+02d", power);
 		}
 		break;
 	    }
@@ -781,12 +786,12 @@ gprintf(
 		       Gibi   3
 		       Mebi   2
 		       kibi   1   */
-		    sprintf(dest, temp, " kMGTPEZY"[power]);
+		    snprintf(dest, remaining_space, temp, " kMGTPEZY"[power]);
 		} else if (power > 8) {
 		    /* for the larger values, print x2^{10}Gi for example */
-		    sprintf(dest, "x2^{%d}Yi", power-8);
+		    snprintf(dest, remaining_space, "x2^{%d}Yi", power-8);
 		} else if (power < 0) {
-		    sprintf(dest, "x2^{%d}", power*10);
+		    snprintf(dest, remaining_space, "x2^{%d}", power*10);
 		}
 
 		break;
@@ -797,7 +802,7 @@ gprintf(
 	    {
 		t[0] = 'f';
 		t[1] = 0;
-		sprintf(dest, temp, x / M_PI);
+		snprintf(dest, remaining_space, temp, x / M_PI);
 		break;
 	    }
 	    /*}}} */
@@ -845,12 +850,15 @@ gprintf(
 	++format;
     } /* for ever */
 
+    /* Copy as much as fits */
+    safe_strncpy(outstring, tempdest, count);
+
     reset_numeric_locale();
 }
 
 /*}}} */
-#ifdef HAVE_SNPRINTF
-# undef sprintf
+#ifndef HAVE_SNPRINTF
+# undef snprintf
 #endif
 
 /* some macros for the error and warning functions below
@@ -1010,7 +1018,9 @@ int_error(int t_num, const char str[], va_dcl)
     df_reset_after_error();
     eval_reset_after_error();
     clause_reset_after_error();
+    parse_reset_after_error();
     scanning_range_in_progress = FALSE;
+    inside_zoom = FALSE;
 
     /* Load error state variables */
     update_gpval_variables(2);
@@ -1328,3 +1338,77 @@ utf8toulong (unsigned long * wch, const char ** str)
   *wch = INVALID_UTF8;
   return FALSE;
 }
+
+/*
+ * Returns number of (possibly multi-byte) characters in a UTF-8 string
+ */
+size_t
+strlen_utf8(const char *s)
+{
+    int i = 0, j = 0;
+    while (s[i]) {
+	if ((s[i] & 0xc0) != 0x80) j++;
+	i++;
+    }
+    return j;
+}
+
+size_t
+gp_strlen(const char *s)
+{
+    if (encoding == S_ENC_UTF8)
+	return strlen_utf8(s);
+    else
+	return strlen(s);
+}
+
+/*
+ * Returns a pointer to the Nth character of s
+ * or a pointer to the trailing \0 if N is too large
+ */
+char *
+utf8_strchrn(const char *s, int N)
+{
+    int i = 0, j = 0;
+
+    if (N <= 0)
+	return (char *)s;
+    while (s[i]) {
+	if ((s[i] & 0xc0) != 0x80) {
+	    if (j++ == N) return (char *)&s[i];
+	}
+	i++;
+    }
+    return (char *)&s[i];
+}
+
+char *
+gp_strchrn(const char *s, int N)
+{
+    if (encoding == S_ENC_UTF8)
+	return utf8_strchrn(s,N);
+    else
+	return (char *)&s[N];
+}
+
+/* TRUE if strings a and b are identical save for leading or trailing whitespace */
+TBOOLEAN
+streq(const char *a, const char *b)
+{
+    int enda, endb;
+
+    while (isspace(*a))
+	a++;
+    while (isspace(*b))
+	b++;
+
+    enda = strlen(a) - 1;
+    endb = strlen(b) - 1;
+
+    while (isspace(a[enda]))
+	enda--;
+    while (isspace(b[endb]))
+	endb--;
+
+    return (enda == endb) ? !strncmp(a,b,++enda) : FALSE;
+}
