@@ -1,5 +1,5 @@
 /*
- * $Id: winmain.h,v 1.7 2008/10/10 22:23:20 mikulik Exp $
+ * $Id: winmain.h,v 1.10.2.1 2013/04/05 16:39:49 markisch Exp $
  */
 
 /* GNUPLOT - win/winmain.h */
@@ -49,17 +49,23 @@
 #include "wgnuplib.h"
 
 extern TW textwin;
-extern GW graphwin;
+extern LPGW graphwin;
+extern LPGW listgraphs;
 extern PW pausewin;
 extern MW menuwin;
 
+extern HWND help_window;
 extern LPSTR winhelpname;
 extern LPSTR szModuleName;
 extern LPSTR szPackageDir;
+extern LPSTR szMenuName;
 
 int Pause __PROTO((LPSTR str));
 void screen_dump __PROTO((void));
 void kill_pending_Pause_dialog __PROTO((void));
 void win_sleep __PROTO((DWORD dwMilliSeconds));
+TBOOLEAN WinWindowOpened(void);
+void WinPersistTextClose(void);
+void WinMessageLoop(void);
 
 #endif /* GNUPLOT_WINMAIN_H */
